@@ -1,23 +1,24 @@
 /*
  * Copyright © 2008 Kristian Høgsberg
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <config.h>
@@ -42,7 +43,8 @@
 
 #include <wayland-client.h>
 
-#include "../shared/config-parser.h"
+#include "shared/config-parser.h"
+#include "shared/helpers.h"
 #include "window.h"
 
 static int option_fullscreen;
@@ -132,28 +134,28 @@ utf8_next_char(struct utf8_state_machine *machine, unsigned char c)
 	case utf8state_reject:
 		machine->s.ch = 0;
 		machine->len = 0;
-		if(c == 0xC0 || c == 0xC1) {
+		if (c == 0xC0 || c == 0xC1) {
 			/* overlong encoding, reject */
 			machine->state = utf8state_reject;
-		} else if((c & 0x80) == 0) {
+		} else if ((c & 0x80) == 0) {
 			/* single byte, accept */
 			machine->s.byte[machine->len++] = c;
 			machine->state = utf8state_accept;
 			machine->unicode = c;
-		} else if((c & 0xC0) == 0x80) {
+		} else if ((c & 0xC0) == 0x80) {
 			/* parser out of sync, ignore byte */
 			machine->state = utf8state_start;
-		} else if((c & 0xE0) == 0xC0) {
+		} else if ((c & 0xE0) == 0xC0) {
 			/* start of two byte sequence */
 			machine->s.byte[machine->len++] = c;
 			machine->state = utf8state_expect1;
 			machine->unicode = c & 0x1f;
-		} else if((c & 0xF0) == 0xE0) {
+		} else if ((c & 0xF0) == 0xE0) {
 			/* start of three byte sequence */
 			machine->s.byte[machine->len++] = c;
 			machine->state = utf8state_expect2;
 			machine->unicode = c & 0x0f;
-		} else if((c & 0xF8) == 0xF0) {
+		} else if ((c & 0xF8) == 0xF0) {
 			/* start of four byte sequence */
 			machine->s.byte[machine->len++] = c;
 			machine->state = utf8state_expect3;
@@ -166,7 +168,7 @@ utf8_next_char(struct utf8_state_machine *machine, unsigned char c)
 	case utf8state_expect3:
 		machine->s.byte[machine->len++] = c;
 		machine->unicode = (machine->unicode << 6) | (c & 0x3f);
-		if((c & 0xC0) == 0x80) {
+		if ((c & 0xC0) == 0x80) {
 			/* all good, continue */
 			machine->state = utf8state_expect2;
 		} else {
@@ -177,7 +179,7 @@ utf8_next_char(struct utf8_state_machine *machine, unsigned char c)
 	case utf8state_expect2:
 		machine->s.byte[machine->len++] = c;
 		machine->unicode = (machine->unicode << 6) | (c & 0x3f);
-		if((c & 0xC0) == 0x80) {
+		if ((c & 0xC0) == 0x80) {
 			/* all good, continue */
 			machine->state = utf8state_expect1;
 		} else {
@@ -188,7 +190,7 @@ utf8_next_char(struct utf8_state_machine *machine, unsigned char c)
 	case utf8state_expect1:
 		machine->s.byte[machine->len++] = c;
 		machine->unicode = (machine->unicode << 6) | (c & 0x3f);
-		if((c & 0xC0) == 0x80) {
+		if ((c & 0xC0) == 0x80) {
 			/* all good, accept */
 			machine->state = utf8state_accept;
 		} else {
@@ -658,7 +660,7 @@ terminal_scroll_window(struct terminal *terminal, int d)
 	// scrolling range is inclusive
 	window_height = terminal->margin_bottom - terminal->margin_top + 1;
 	d = d % (window_height + 1);
-	if(d < 0) {
+	if (d < 0) {
 		d = 0 - d;
 		to_row = terminal->margin_bottom;
 		from_row = terminal->margin_bottom - d;
@@ -699,7 +701,7 @@ terminal_scroll_window(struct terminal *terminal, int d)
 static void
 terminal_scroll(struct terminal *terminal, int d)
 {
-	if(terminal->margin_top == 0 && terminal->margin_bottom == terminal->height - 1)
+	if (terminal->margin_top == 0 && terminal->margin_bottom == terminal->height - 1)
 		terminal_scroll_buffer(terminal, d);
 	else
 		terminal_scroll_window(terminal, d);
@@ -1555,17 +1557,17 @@ handle_escape(struct terminal *terminal)
 		}
 		break;
 	case 'h':    /* SM */
-		for(i = 0; i < 10 && set[i]; i++) {
+		for (i = 0; i < 10 && set[i]; i++) {
 			handle_term_parameter(terminal, args[i], 1);
 		}
 		break;
 	case 'l':    /* RM */
-		for(i = 0; i < 10 && set[i]; i++) {
+		for (i = 0; i < 10 && set[i]; i++) {
 			handle_term_parameter(terminal, args[i], 0);
 		}
 		break;
 	case 'm':    /* SGR */
-		for(i = 0; i < 10; i++) {
+		for (i = 0; i < 10; i++) {
 			if (i <= 7 && set[i] && set[i + 1] &&
 				set[i + 2] && args[i + 1] == 5)
 			{
@@ -1577,9 +1579,9 @@ handle_escape(struct terminal *terminal)
 					break;
 				}
 			}
-			if(set[i]) {
+			if (set[i]) {
 				handle_sgr(terminal, args[i]);
-			} else if(i == 0) {
+			} else if (i == 0) {
 				handle_sgr(terminal, 0);
 				break;
 			} else {
@@ -1600,7 +1602,7 @@ handle_escape(struct terminal *terminal)
 		}
  		break;
 	case 'r':
-		if(!set[0]) {
+		if (!set[0]) {
 			terminal->margin_top = 0;
 			terminal->margin_bottom = terminal->height-1;
 			terminal->row = 0;
@@ -1612,14 +1614,14 @@ handle_escape(struct terminal *terminal)
 			bottom = (set[1] ? args[1] : 1) - 1;
 			bottom = bottom < 0 ? 0 :
 			         (bottom >= terminal->height ? terminal->height - 1 : bottom);
-			if(bottom > top) {
+			if (bottom > top) {
 				terminal->margin_top = top;
 				terminal->margin_bottom = bottom;
 			} else {
 				terminal->margin_top = 0;
 				terminal->margin_bottom = terminal->height-1;
 			}
-			if(terminal->origin_mode)
+			if (terminal->origin_mode)
 				terminal->row = terminal->margin_top;
 			else
 				terminal->row = 0;
@@ -1689,7 +1691,7 @@ handle_non_csi_escape(struct terminal *terminal, char code)
 	switch(code) {
 	case 'M':    /* RI */
 		terminal->row -= 1;
-		if(terminal->row < terminal->margin_top) {
+		if (terminal->row < terminal->margin_top) {
 			terminal->row = terminal->margin_top;
 			terminal_scroll(terminal, -1);
 		}
@@ -1699,7 +1701,7 @@ handle_non_csi_escape(struct terminal *terminal, char code)
 		// fallthrough
 	case 'D':    /* IND */
 		terminal->row += 1;
-		if(terminal->row > terminal->margin_bottom) {
+		if (terminal->row > terminal->margin_bottom) {
 			terminal->row = terminal->margin_bottom;
 			terminal_scroll(terminal, +1);
 		}
@@ -1751,7 +1753,7 @@ handle_special_escape(struct terminal *terminal, char special, char code)
 			/* fill with 'E', no cheap way to do this */
 			memset(terminal->data, 0, terminal->data_pitch * terminal->height);
 			numChars = terminal->width * terminal->height;
-			for(i = 0; i < numChars; i++) {
+			for (i = 0; i < numChars; i++) {
 				terminal->data[i].byte[0] = 'E';
 			}
 			break;
@@ -1839,19 +1841,19 @@ handle_sgr(struct terminal *terminal, int code)
 		terminal->curr_attr.bg = terminal->color_scheme->default_attr.bg;
 		break;
 	default:
-		if(code >= 30 && code <= 37) {
+		if (code >= 30 && code <= 37) {
 			terminal->curr_attr.fg = code - 30;
 			if (terminal->curr_attr.a & ATTRMASK_BOLD)
 				terminal->curr_attr.fg += 8;
-		} else if(code >= 40 && code <= 47) {
+		} else if (code >= 40 && code <= 47) {
 			terminal->curr_attr.bg = code - 40;
 		} else if (code >= 90 && code <= 97) {
 			terminal->curr_attr.fg = code - 90 + 8;
 		} else if (code >= 100 && code <= 107) {
 			terminal->curr_attr.bg = code - 100 + 8;
-		} else if(code >= 256 && code < 512) {
+		} else if (code >= 256 && code < 512) {
 			terminal->curr_attr.fg = code - 256;
-		} else if(code >= 512 && code < 768) {
+		} else if (code >= 512 && code < 768) {
 			terminal->curr_attr.bg = code - 512;
 		} else {
 			fprintf(stderr, "Unknown SGR code: %d\n", code);
@@ -2577,7 +2579,7 @@ static int wordsep(int ch)
 {
 	const char extra[] = "-,./?%&#:_=+@~";
 
-	if (ch > 127)
+	if (ch > 127 || ch < 0)
 		return 1;
 
 	return ch == 0 || !(isalpha(ch) || isdigit(ch) || strchr(extra, ch));
