@@ -1,23 +1,26 @@
 /*
  * Copyright © 2013 Hardening <rdp.effort@gmail.com>
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -62,6 +65,7 @@
 #include <freerdp/locale/keyboard.h>
 #include <winpr/input.h>
 
+#include "shared/helpers.h"
 #include "compositor.h"
 #include "pixman-renderer.h"
 
@@ -69,7 +73,7 @@
 #define DEFAULT_AXIS_STEP_DISTANCE wl_fixed_from_int(10)
 #define RDP_MODE_FREQ 60 * 1000
 
-struct rdp_compositor_config {
+struct rdp_backend_config {
 	int width;
 	int height;
 	char *bind_address;
@@ -83,8 +87,9 @@ struct rdp_compositor_config {
 
 struct rdp_output;
 
-struct rdp_compositor {
-	struct weston_compositor base;
+struct rdp_backend {
+	struct weston_backend base;
+	struct weston_compositor *compositor;
 
 	freerdp_listener *listener;
 	struct wl_event_source *listener_events[MAX_FREERDP_FDS];
@@ -121,7 +126,7 @@ struct rdp_output {
 struct rdp_peer_context {
 	rdpContext _p;
 
-	struct rdp_compositor *rdpCompositor;
+	struct rdp_backend *rdpBackend;
 	struct wl_event_source *events[MAX_FREERDP_FDS];
 	RFX_CONTEXT *rfx_context;
 	wStream *encode_stream;
@@ -133,7 +138,8 @@ struct rdp_peer_context {
 typedef struct rdp_peer_context RdpPeerContext;
 
 static void
-rdp_compositor_config_init(struct rdp_compositor_config *config) {
+rdp_backend_config_init(struct rdp_backend_config *config)
+{
 	config->width = 640;
 	config->height = 480;
 	config->bind_address = NULL;
@@ -245,7 +251,8 @@ rdp_peer_refresh_nsc(pixman_region32_t *damage, pixman_image_t *image, freerdp_p
 }
 
 static void
-pixman_image_flipped_subrect(const pixman_box32_t *rect, pixman_image_t *img, BYTE *dest) {
+pixman_image_flipped_subrect(const pixman_box32_t *rect, pixman_image_t *img, BYTE *dest)
+{
 	int stride = pixman_image_get_stride(img);
 	int h;
 	int toCopy = (rect->x2 - rect->x1) * 4;
@@ -319,7 +326,7 @@ static void
 rdp_peer_refresh_region(pixman_region32_t *region, freerdp_peer *peer)
 {
 	RdpPeerContext *context = (RdpPeerContext *)peer->context;
-	struct rdp_output *output = context->rdpCompositor->output;
+	struct rdp_output *output = context->rdpBackend->output;
 	rdpSettings *settings = peer->settings;
 
 	if (settings->RemoteFxCodec)
@@ -388,7 +395,8 @@ finish_frame_handler(void *data)
 }
 
 static struct weston_mode *
-rdp_insert_new_mode(struct weston_output *output, int width, int height, int rate) {
+rdp_insert_new_mode(struct weston_output *output, int width, int height, int rate)
+{
 	struct weston_mode *ret;
 	ret = zalloc(sizeof *ret);
 	if (!ret)
@@ -401,7 +409,8 @@ rdp_insert_new_mode(struct weston_output *output, int width, int height, int rat
 }
 
 static struct weston_mode *
-ensure_matching_mode(struct weston_output *output, struct weston_mode *target) {
+ensure_matching_mode(struct weston_output *output, struct weston_mode *target)
+{
 	struct weston_mode *local;
 
 	wl_list_for_each(local, &output->mode_list, link) {
@@ -413,7 +422,8 @@ ensure_matching_mode(struct weston_output *output, struct weston_mode *target) {
 }
 
 static int
-rdp_switch_mode(struct weston_output *output, struct weston_mode *target_mode) {
+rdp_switch_mode(struct weston_output *output, struct weston_mode *target_mode)
+{
 	struct rdp_output *rdpOutput = container_of(output, struct rdp_output, base);
 	struct rdp_peers_item *rdpPeer;
 	rdpSettings *settings;
@@ -463,7 +473,7 @@ rdp_switch_mode(struct weston_output *output, struct weston_mode *target_mode) {
 }
 
 static int
-rdp_compositor_create_output(struct rdp_compositor *c, int width, int height)
+rdp_backend_create_output(struct rdp_backend *b, int width, int height)
 {
 	struct rdp_output *output;
 	struct wl_event_loop *loop;
@@ -487,7 +497,7 @@ rdp_compositor_create_output(struct rdp_compositor *c, int width, int height)
 		goto out_free_output;
 
 	output->base.current_mode = output->base.native_mode = currentMode;
-	weston_output_init(&output->base, &c->base, 0, 0, width, height,
+	weston_output_init(&output->base, b->compositor, 0, 0, width, height,
 			   WL_OUTPUT_TRANSFORM_NORMAL, 1);
 
 	output->base.make = "weston";
@@ -504,7 +514,7 @@ rdp_compositor_create_output(struct rdp_compositor *c, int width, int height)
 	if (pixman_renderer_output_create(&output->base) < 0)
 		goto out_shadow_surface;
 
-	loop = wl_display_get_event_loop(c->base.wl_display);
+	loop = wl_display_get_event_loop(b->compositor->wl_display);
 	output->finish_frame_timer = wl_event_loop_add_timer(loop, finish_frame_handler, output);
 
 	output->base.start_repaint_loop = rdp_output_start_repaint_loop;
@@ -514,9 +524,9 @@ rdp_compositor_create_output(struct rdp_compositor *c, int width, int height)
 	output->base.set_backlight = NULL;
 	output->base.set_dpms = NULL;
 	output->base.switch_mode = rdp_switch_mode;
-	c->output = output;
+	b->output = output;
 
-	weston_compositor_add_output(&c->base, &output->base);
+	weston_compositor_add_output(b->compositor, &output->base);
 	return 0;
 
 out_shadow_surface:
@@ -536,19 +546,30 @@ rdp_restore(struct weston_compositor *ec)
 static void
 rdp_destroy(struct weston_compositor *ec)
 {
-	weston_compositor_shutdown(ec);
+	struct rdp_backend *b = (struct rdp_backend *) ec->backend;
+	int i;
 
-	free(ec);
+	weston_compositor_shutdown(ec);
+	for (i = 0; i < MAX_FREERDP_FDS; i++)
+		if (b->listener_events[i])
+			wl_event_source_remove(b->listener_events[i]);
+
+	freerdp_listener_free(b->listener);
+
+	free(b->server_cert);
+	free(b->server_key);
+	free(b->rdp_key);
+	free(b);
 }
 
 static
-int rdp_listener_activity(int fd, uint32_t mask, void *data) {
+int rdp_listener_activity(int fd, uint32_t mask, void *data)
+{
 	freerdp_listener* instance = (freerdp_listener *)data;
 
 	if (!(mask & WL_EVENT_READABLE))
 		return 0;
-	if (!instance->CheckFileDescriptor(instance))
-	{
+	if (!instance->CheckFileDescriptor(instance)) {
 		weston_log("failed to check FreeRDP file descriptor\n");
 		return -1;
 	}
@@ -556,7 +577,8 @@ int rdp_listener_activity(int fd, uint32_t mask, void *data) {
 }
 
 static
-int rdp_implant_listener(struct rdp_compositor *c, freerdp_listener* instance) {
+int rdp_implant_listener(struct rdp_backend *b, freerdp_listener* instance)
+{
 	int i, fd;
 	int rcount = 0;
 	void* rfds[MAX_FREERDP_FDS];
@@ -567,15 +589,15 @@ int rdp_implant_listener(struct rdp_compositor *c, freerdp_listener* instance) {
 		return -1;
 	}
 
-	loop = wl_display_get_event_loop(c->base.wl_display);
+	loop = wl_display_get_event_loop(b->compositor->wl_display);
 	for (i = 0; i < rcount; i++) {
 		fd = (int)(long)(rfds[i]);
-		c->listener_events[i] = wl_event_loop_add_fd(loop, fd, WL_EVENT_READABLE,
+		b->listener_events[i] = wl_event_loop_add_fd(loop, fd, WL_EVENT_READABLE,
 				rdp_listener_activity, instance);
 	}
 
-	for( ; i < MAX_FREERDP_FDS; i++)
-		c->listener_events[i] = 0;
+	for ( ; i < MAX_FREERDP_FDS; i++)
+		b->listener_events[i] = 0;
 	return 0;
 }
 
@@ -610,7 +632,7 @@ rdp_peer_context_free(freerdp_peer* client, RdpPeerContext* context)
 		return;
 
 	wl_list_remove(&context->item.link);
-	for(i = 0; i < MAX_FREERDP_FDS; i++) {
+	for (i = 0; i < MAX_FREERDP_FDS; i++) {
 		if (context->events[i])
 			wl_event_source_remove(context->events[i]);
 	}
@@ -629,7 +651,8 @@ rdp_peer_context_free(freerdp_peer* client, RdpPeerContext* context)
 
 
 static int
-rdp_client_activity(int fd, uint32_t mask, void *data) {
+rdp_client_activity(int fd, uint32_t mask, void *data)
+{
 	freerdp_peer* client = (freerdp_peer *)data;
 
 	if (!client->CheckFileDescriptor(client)) {
@@ -789,7 +812,7 @@ static BOOL
 xf_peer_activate(freerdp_peer* client)
 {
 	RdpPeerContext *peerCtx;
-	struct rdp_compositor *c;
+	struct rdp_backend *b;
 	struct rdp_output *output;
 	rdpSettings *settings;
 	rdpPointerUpdate *pointer;
@@ -804,9 +827,9 @@ xf_peer_activate(freerdp_peer* client)
 
 
 	peerCtx = (RdpPeerContext *)client->context;
-	c = peerCtx->rdpCompositor;
+	b = peerCtx->rdpBackend;
 	peersItem = &peerCtx->item;
-	output = c->output;
+	output = b->output;
 	settings = client->settings;
 
 	if (!settings->SurfaceCommandsEnabled) {
@@ -817,7 +840,7 @@ xf_peer_activate(freerdp_peer* client)
 	if (output->base.width != (int)settings->DesktopWidth ||
 			output->base.height != (int)settings->DesktopHeight)
 	{
-		if (c->no_clients_resize) {
+		if (b->no_clients_resize) {
 			/* RDP peers don't dictate their resolution to weston */
 			if (!settings->DesktopResize) {
 				/* peer does not support desktop resize */
@@ -861,7 +884,7 @@ xf_peer_activate(freerdp_peer* client)
 	memset(&xkbRuleNames, 0, sizeof(xkbRuleNames));
 	if (settings->KeyboardType <= 7)
 		xkbRuleNames.model = rdp_keyboard_types[settings->KeyboardType];
-	for(i = 0; rdp_keyboards[i].rdpLayoutCode; i++) {
+	for (i = 0; rdp_keyboards[i].rdpLayoutCode; i++) {
 		if (rdp_keyboards[i].rdpLayoutCode == settings->KeyboardLayout) {
 			xkbRuleNames.layout = rdp_keyboards[i].xkbLayout;
 			xkbRuleNames.variant = rdp_keyboards[i].xkbVariant;
@@ -887,7 +910,7 @@ xf_peer_activate(freerdp_peer* client)
 	else
 		snprintf(seat_name, sizeof(seat_name), "RDP peer @%s", settings->ClientAddress);
 
-	weston_seat_init(&peersItem->seat, &c->base, seat_name);
+	weston_seat_init(&peersItem->seat, b->compositor, seat_name);
 	weston_seat_init_keyboard(&peersItem->seat, keymap);
 	weston_seat_init_pointer(&peersItem->seat);
 
@@ -918,14 +941,15 @@ static BOOL xf_peer_post_connect(freerdp_peer *client)
 }
 
 static FREERDP_CB_RET_TYPE
-xf_mouseEvent(rdpInput *input, UINT16 flags, UINT16 x, UINT16 y) {
+xf_mouseEvent(rdpInput *input, UINT16 flags, UINT16 x, UINT16 y)
+{
 	wl_fixed_t wl_x, wl_y, axis;
 	RdpPeerContext *peerContext = (RdpPeerContext *)input->context;
 	struct rdp_output *output;
 	uint32_t button = 0;
 
 	if (flags & PTR_FLAGS_MOVE) {
-		output = peerContext->rdpCompositor->output;
+		output = peerContext->rdpBackend->output;
 		if (x < output->base.width && y < output->base.height) {
 			wl_x = wl_fixed_from_int((int)x);
 			wl_y = wl_fixed_from_int((int)y);
@@ -967,12 +991,13 @@ xf_mouseEvent(rdpInput *input, UINT16 flags, UINT16 x, UINT16 y) {
 }
 
 static FREERDP_CB_RET_TYPE
-xf_extendedMouseEvent(rdpInput *input, UINT16 flags, UINT16 x, UINT16 y) {
+xf_extendedMouseEvent(rdpInput *input, UINT16 flags, UINT16 x, UINT16 y)
+{
 	wl_fixed_t wl_x, wl_y;
 	RdpPeerContext *peerContext = (RdpPeerContext *)input->context;
 	struct rdp_output *output;
 
-	output = peerContext->rdpCompositor->output;
+	output = peerContext->rdpBackend->output;
 	if (x < output->base.width && y < output->base.height) {
 		wl_x = wl_fixed_from_int((int)x);
 		wl_y = wl_fixed_from_int((int)y);
@@ -989,7 +1014,7 @@ xf_input_synchronize_event(rdpInput *input, UINT32 flags)
 {
 	freerdp_peer *client = input->context->peer;
 	RdpPeerContext *peerCtx = (RdpPeerContext *)input->context;
-	struct rdp_output *output = peerCtx->rdpCompositor->output;
+	struct rdp_output *output = peerCtx->rdpBackend->output;
 	pixman_box32_t box;
 	pixman_region32_t damage;
 
@@ -1032,7 +1057,7 @@ xf_input_keyboard_event(rdpInput *input, UINT16 flags, UINT16 code)
 			full_code |= KBD_FLAGS_EXTENDED;
 
 		vk_code = GetVirtualKeyCodeFromVirtualScanCode(full_code, 4);
-		if(flags & KBD_FLAGS_EXTENDED)
+		if (flags & KBD_FLAGS_EXTENDED)
 			vk_code |= KBDEXT;
 
 		scan_code = GetKeycodeFromVirtualKeyCode(vk_code, KEYCODE_TYPE_EVDEV);
@@ -1055,7 +1080,8 @@ xf_input_unicode_keyboard_event(rdpInput *input, UINT16 flags, UINT16 code)
 
 
 static FREERDP_CB_RET_TYPE
-xf_suppress_output(rdpContext *context, BYTE allow, RECTANGLE_16 *area) {
+xf_suppress_output(rdpContext *context, BYTE allow, RECTANGLE_16 *area)
+{
 	RdpPeerContext *peerContext = (RdpPeerContext *)context;
 
 	if (allow)
@@ -1067,7 +1093,7 @@ xf_suppress_output(rdpContext *context, BYTE allow, RECTANGLE_16 *area) {
 }
 
 static int
-rdp_peer_init(freerdp_peer *client, struct rdp_compositor *c)
+rdp_peer_init(freerdp_peer *client, struct rdp_backend *b)
 {
 	int rcount = 0;
 	void *rfds[MAX_FREERDP_FDS];
@@ -1083,22 +1109,25 @@ rdp_peer_init(freerdp_peer *client, struct rdp_compositor *c)
 	freerdp_peer_context_new(client);
 
 	peerCtx = (RdpPeerContext *) client->context;
-	peerCtx->rdpCompositor = c;
-
-	client->Initialize(client);
+	peerCtx->rdpBackend = b;
 
 	settings = client->settings;
 	/* configure security settings */
-	if (c->rdp_key)
-		settings->RdpKeyFile = strdup(c->rdp_key);
-	if (c->tls_enabled) {
-		settings->CertificateFile = strdup( c->server_cert);
-		settings->PrivateKeyFile = strdup(c->server_key);
+	if (b->rdp_key)
+		settings->RdpKeyFile = strdup(b->rdp_key);
+	if (b->tls_enabled) {
+		settings->CertificateFile = strdup(b->server_cert);
+		settings->PrivateKeyFile = strdup(b->server_key);
 	} else {
 		settings->TlsSecurity = FALSE;
 	}
 	settings->NlaSecurity = FALSE;
 
+	if (!client->Initialize(client)) {
+		weston_log("peer initialization failed\n");
+		goto error_initialize;
+	}
+
 	settings->OsMajorType = OSMAJORTYPE_UNIX;
 	settings->OsMinorType = OSMINORTYPE_PSEUDO_XSERVER;
 	settings->ColorDepth = 32;
@@ -1123,11 +1152,11 @@ rdp_peer_init(freerdp_peer *client, struct rdp_compositor *c)
 
 	if (!client->GetFileDescriptor(client, rfds, &rcount)) {
 		weston_log("unable to retrieve client fds\n");
-		return -1;
+		goto error_initialize;
 	}
 
-	loop = wl_display_get_event_loop(c->base.wl_display);
-	for(i = 0; i < rcount; i++) {
+	loop = wl_display_get_event_loop(b->compositor->wl_display);
+	for (i = 0; i < rcount; i++) {
 		fd = (int)(long)(rfds[i]);
 
 		peerCtx->events[i] = wl_event_loop_add_fd(loop, fd, WL_EVENT_READABLE,
@@ -1136,75 +1165,77 @@ rdp_peer_init(freerdp_peer *client, struct rdp_compositor *c)
 	for ( ; i < MAX_FREERDP_FDS; i++)
 		peerCtx->events[i] = 0;
 
-	wl_list_insert(&c->output->peers, &peerCtx->item.link);
+	wl_list_insert(&b->output->peers, &peerCtx->item.link);
 	return 0;
+
+error_initialize:
+	client->Close(client);
+	return -1;
 }
 
 
 static FREERDP_CB_RET_TYPE
 rdp_incoming_peer(freerdp_listener *instance, freerdp_peer *client)
 {
-	struct rdp_compositor *c = (struct rdp_compositor *)instance->param4;
-	if (rdp_peer_init(client, c) < 0) {
-		weston_log("error when treating incoming peer");
+	struct rdp_backend *b = (struct rdp_backend *)instance->param4;
+	if (rdp_peer_init(client, b) < 0) {
+		weston_log("error when treating incoming peer\n");
 		FREERDP_CB_RETURN(FALSE);
 	}
 
 	FREERDP_CB_RETURN(TRUE);
 }
 
-static struct weston_compositor *
-rdp_compositor_create(struct wl_display *display,
-		struct rdp_compositor_config *config,
-		int *argc, char *argv[], struct weston_config *wconfig)
+static struct rdp_backend *
+rdp_backend_create(struct weston_compositor *compositor,
+		   struct rdp_backend_config *config,
+		   int *argc, char *argv[], struct weston_config *wconfig)
 {
-	struct rdp_compositor *c;
+	struct rdp_backend *b;
 	char *fd_str;
 	int fd;
 
-	c = zalloc(sizeof *c);
-	if (c == NULL)
+	b = zalloc(sizeof *b);
+	if (b == NULL)
 		return NULL;
 
-	if (weston_compositor_init(&c->base, display, argc, argv, wconfig) < 0)
-		goto err_free;
-
-	c->base.destroy = rdp_destroy;
-	c->base.restore = rdp_restore;
-	c->rdp_key = config->rdp_key ? strdup(config->rdp_key) : NULL;
-	c->no_clients_resize = config->no_clients_resize;
+	b->compositor = compositor;
+	b->base.destroy = rdp_destroy;
+	b->base.restore = rdp_restore;
+	b->rdp_key = config->rdp_key ? strdup(config->rdp_key) : NULL;
+	b->no_clients_resize = config->no_clients_resize;
 
 	/* activate TLS only if certificate/key are available */
 	if (config->server_cert && config->server_key) {
 		weston_log("TLS support activated\n");
-		c->server_cert = strdup(config->server_cert);
-		c->server_key = strdup(config->server_key);
-		if (!c->server_cert || !c->server_key)
+		b->server_cert = strdup(config->server_cert);
+		b->server_key = strdup(config->server_key);
+		if (!b->server_cert || !b->server_key)
 			goto err_free_strings;
-		c->tls_enabled = 1;
+		b->tls_enabled = 1;
 	}
 
-	if (weston_compositor_set_presentation_clock_software(&c->base) < 0)
+	if (weston_compositor_set_presentation_clock_software(compositor) < 0)
 		goto err_compositor;
 
-	if (pixman_renderer_init(&c->base) < 0)
+	if (pixman_renderer_init(compositor) < 0)
 		goto err_compositor;
 
-	if (rdp_compositor_create_output(c, config->width, config->height) < 0)
+	if (rdp_backend_create_output(b, config->width, config->height) < 0)
 		goto err_compositor;
 
-	c->base.capabilities |= WESTON_CAP_ARBITRARY_MODES;
+	compositor->capabilities |= WESTON_CAP_ARBITRARY_MODES;
 
-	if(!config->env_socket) {
-		c->listener = freerdp_listener_new();
-		c->listener->PeerAccepted = rdp_incoming_peer;
-		c->listener->param4 = c;
-		if (!c->listener->Open(c->listener, config->bind_address, config->port)) {
+	if (!config->env_socket) {
+		b->listener = freerdp_listener_new();
+		b->listener->PeerAccepted = rdp_incoming_peer;
+		b->listener->param4 = b;
+		if (!b->listener->Open(b->listener, config->bind_address, config->port)) {
 			weston_log("unable to bind rdp socket\n");
 			goto err_listener;
 		}
 
-		if (rdp_implant_listener(c, c->listener) < 0)
+		if (rdp_implant_listener(b, b->listener) < 0)
 			goto err_compositor;
 	} else {
 		/* get the socket from RDP_FD var */
@@ -1215,36 +1246,34 @@ rdp_compositor_create(struct wl_display *display,
 		}
 
 		fd = strtoul(fd_str, NULL, 10);
-		if (rdp_peer_init(freerdp_peer_new(fd), c))
+		if (rdp_peer_init(freerdp_peer_new(fd), b))
 			goto err_output;
 	}
 
-	return &c->base;
+	compositor->backend = &b->base;
+	return b;
 
 err_listener:
-	freerdp_listener_free(c->listener);
+	freerdp_listener_free(b->listener);
 err_output:
-	weston_output_destroy(&c->output->base);
+	weston_output_destroy(&b->output->base);
 err_compositor:
-	weston_compositor_shutdown(&c->base);
+	weston_compositor_shutdown(compositor);
 err_free_strings:
-	if (c->rdp_key)
-		free(c->rdp_key);
-	if (c->server_cert)
-		free(c->server_cert);
-	if (c->server_key)
-		free(c->server_key);
-err_free:
-	free(c);
+	free(b->rdp_key);
+	free(b->server_cert);
+	free(b->server_key);
+	free(b);
 	return NULL;
 }
 
-WL_EXPORT struct weston_compositor *
-backend_init(struct wl_display *display, int *argc, char *argv[],
+WL_EXPORT int
+backend_init(struct weston_compositor *compositor, int *argc, char *argv[],
 	     struct weston_config *wconfig)
 {
-	struct rdp_compositor_config config;
-	rdp_compositor_config_init(&config);
+	struct rdp_backend *b;
+	struct rdp_backend_config config;
+	rdp_backend_config_init(&config);
 	int major, minor, revision;
 
 	freerdp_get_version(&major, &minor, &revision);
@@ -1266,8 +1295,11 @@ backend_init(struct wl_display *display, int *argc, char *argv[],
 	if (!config.rdp_key && (!config.server_cert || !config.server_key)) {
 		weston_log("the RDP compositor requires keys and an optional certificate for RDP or TLS security ("
 				"--rdp4-key or --rdp-tls-cert/--rdp-tls-key)\n");
-		return NULL;
+		return -1;
 	}
 
-	return rdp_compositor_create(display, &config, argc, argv, wconfig);
+	b = rdp_backend_create(compositor, &config, argc, argv, wconfig);
+	if (b == NULL)
+		return -1;
+	return 0;
 }
