@@ -1,23 +1,26 @@
 /*
  * Copyright © 2012-2013 Raspberry Pi Foundation
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -34,6 +37,7 @@
 
 #include "compositor.h"
 #include "rpi-renderer.h"
+#include "shared/helpers.h"
 
 #ifdef ENABLE_EGL
 #include <EGL/egl.h>
@@ -149,7 +153,6 @@ struct rpir_output {
 	DISPMANX_DISPLAY_HANDLE_T display;
 
 	DISPMANX_UPDATE_HANDLE_T update;
-	struct weston_matrix matrix;
 
 	/* all Elements currently on screen */
 	struct wl_list view_list; /* struct rpir_surface::link */
@@ -674,7 +677,6 @@ rpir_view_compute_rects(struct rpir_view *view,
 			VC_IMAGE_TRANSFORM_T *flipmask)
 {
 	struct weston_output *output_base = view->view->surface->output;
-	struct rpir_output *output = to_rpir_output(output_base);
 	struct weston_matrix matrix = view->view->transform.matrix;
 	VC_IMAGE_TRANSFORM_T flipt = 0;
 	int src_x, src_y;
@@ -701,6 +703,9 @@ rpir_view_compute_rects(struct rpir_view *view,
 		struct weston_buffer *buffer =
 			view->surface->egl_front->buffer_ref.buffer;
 
+		if (!buffer)
+			return -1;
+
 		src_width = buffer->width << 16;
 		src_height = buffer->height << 16;
 	} else {
@@ -708,14 +713,14 @@ rpir_view_compute_rects(struct rpir_view *view,
 		src_height = view->surface->front->height << 16;
 	}
 
-	weston_matrix_multiply(&matrix, &output->matrix);
+	weston_matrix_multiply(&matrix, &output_base->matrix);
 
 #ifdef SURFACE_TRANSFORM
 	if (matrix.type >= WESTON_MATRIX_TRANSFORM_OTHER) {
 #else
 	if (matrix.type >= WESTON_MATRIX_TRANSFORM_ROTATE) {
 #endif
-		warn_bad_matrix(&matrix, &output->matrix,
+		warn_bad_matrix(&matrix, &output_base->matrix,
 				&view->view->transform.matrix);
 	} else {
 		if (matrix.type & WESTON_MATRIX_TRANSFORM_ROTATE) {
@@ -726,7 +731,7 @@ rpir_view_compute_rects(struct rpir_view *view,
 				   fabsf(matrix.d[4]) < 1e-4) {
 				/* no transpose */
 			} else {
-				warn_bad_matrix(&matrix, &output->matrix,
+				warn_bad_matrix(&matrix, &output_base->matrix,
 					&view->view->transform.matrix);
 			}
 		}
@@ -1320,64 +1325,6 @@ rpir_output_dmx_remove_all(struct rpir_output *output,
 	}
 }
 
-static void
-output_compute_matrix(struct weston_output *base)
-{
-	struct rpir_output *output = to_rpir_output(base);
-	struct weston_matrix *matrix = &output->matrix;
-#ifdef SURFACE_TRANSFORM
-	const float half_w = 0.5f * base->width;
-	const float half_h = 0.5f * base->height;
-#endif
-	float mag;
-
-	weston_matrix_init(matrix);
-	weston_matrix_translate(matrix, -base->x, -base->y, 0.0f);
-
-#ifdef SURFACE_TRANSFORM
-	weston_matrix_translate(matrix, -half_w, -half_h, 0.0f);
-	switch (base->transform) {
-	case WL_OUTPUT_TRANSFORM_FLIPPED:
-		weston_matrix_scale(matrix, -1.0f, 1.0f, 1.0f);
-	case WL_OUTPUT_TRANSFORM_NORMAL:
-		/* weston_matrix_rotate_xy(matrix, 1.0f, 0.0f); no-op */
-		weston_matrix_translate(matrix, half_w, half_h, 0.0f);
-		break;
-
-	case WL_OUTPUT_TRANSFORM_FLIPPED_90:
-		weston_matrix_scale(matrix, -1.0f, 1.0f, 1.0f);
-	case WL_OUTPUT_TRANSFORM_90:
-		weston_matrix_rotate_xy(matrix, 0.0f, 1.0f);
-		weston_matrix_translate(matrix, half_h, half_w, 0.0f);
-		break;
-
-	case WL_OUTPUT_TRANSFORM_FLIPPED_180:
-		weston_matrix_scale(matrix, -1.0f, 1.0f, 1.0f);
-	case WL_OUTPUT_TRANSFORM_180:
-		weston_matrix_rotate_xy(matrix, -1.0f, 0.0f);
-		weston_matrix_translate(matrix, half_w, half_h, 0.0f);
-		break;
-
-	case WL_OUTPUT_TRANSFORM_FLIPPED_270:
-		weston_matrix_scale(matrix, -1.0f, 1.0f, 1.0f);
-	case WL_OUTPUT_TRANSFORM_270:
-		weston_matrix_rotate_xy(matrix, 0.0f, -1.0f);
-		weston_matrix_translate(matrix, half_h, half_w, 0.0f);
-		break;
-
-	default:
-		break;
-	}
-#endif
-
-	if (base->zoom.active) {
-		mag = 1.0f / (1.0f - base->zoom.spring_z.current);
-		weston_matrix_translate(matrix, base->zoom.trans_x,
-					base->zoom.trans_y, 0.0f);
-		weston_matrix_scale(matrix, mag, mag, 1.0f);
-	}
-}
-
 /* Note: this won't work right for multiple outputs. A DispmanX Element
  * is tied to one DispmanX Display, i.e. output.
  */
@@ -1394,8 +1341,6 @@ rpi_renderer_repaint_output(struct weston_output *base,
 
 	assert(output->update != DISPMANX_NO_HANDLE);
 
-	output_compute_matrix(base);
-
 	rpi_resource_release(&output->capture_buffer);
 	free(output->capture_data);
 	output->capture_data = NULL;
@@ -1545,7 +1490,7 @@ rpi_renderer_attach(struct weston_surface *base, struct weston_buffer *buffer)
 
 		surface->buffer_type = BUFFER_TYPE_EGL;
 
-		if(surface->egl_back == NULL)
+		if (surface->egl_back == NULL)
 			surface->egl_back = zalloc(sizeof *surface->egl_back);
 
 		weston_buffer_reference(&surface->egl_back->buffer_ref, buffer);
