@@ -2,23 +2,26 @@
  * Copyright © 2008-2011 Kristian Høgsberg
  * Copyright © 2014 Jason Ekstrand
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -32,11 +35,13 @@
 #include <signal.h>
 #include <linux/input.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include <wayland-client.h>
 
 #include "compositor.h"
-#include "../shared/os-compatibility.h"
+#include "shared/helpers.h"
+#include "shared/os-compatibility.h"
 #include "fullscreen-shell-client-protocol.h"
 
 struct shared_output {
@@ -101,6 +106,11 @@ struct ss_shm_buffer {
 	pixman_image_t *pm_image;
 };
 
+struct screen_share {
+	struct weston_compositor *compositor;
+	char *command;
+};
+
 static void
 ss_seat_handle_pointer_enter(void *data, struct wl_pointer *pointer,
 			     uint32_t serial, struct wl_surface *surface,
@@ -163,7 +173,7 @@ static const struct wl_pointer_listener ss_seat_pointer_listener = {
 };
 
 static void
-ss_seat_handle_keymap(void *data, struct wl_keyboard *keyboard,
+ss_seat_handle_keymap(void *data, struct wl_keyboard *wl_keyboard,
 		      uint32_t format, int fd, uint32_t size)
 {
 	struct ss_seat *seat = data;
@@ -180,10 +190,10 @@ ss_seat_handle_keymap(void *data, struct wl_keyboard *keyboard,
 			goto error;
 		}
 
-		keymap = xkb_map_new_from_string(seat->base.compositor->xkb_context,
-						 map_str,
-						 XKB_KEYMAP_FORMAT_TEXT_V1,
-						 0);
+		keymap = xkb_keymap_new_from_string(seat->base.compositor->xkb_context,
+						    map_str,
+						    XKB_KEYMAP_FORMAT_TEXT_V1,
+						    0);
 		munmap(map_str, size);
 
 		if (!keymap) {
@@ -203,13 +213,12 @@ ss_seat_handle_keymap(void *data, struct wl_keyboard *keyboard,
 
 	close(fd);
 
-	if (seat->base.keyboard)
+	if (seat->base.keyboard_device_count)
 		weston_seat_update_keymap(&seat->base, keymap);
 	else
 		weston_seat_init_keyboard(&seat->base, keymap);
 
-	if (keymap)
-		xkb_map_unref(keymap);
+	xkb_keymap_unref(keymap);
 
 	return;
 
@@ -255,13 +264,14 @@ ss_seat_handle_key(void *data, struct wl_keyboard *keyboard,
 }
 
 static void
-ss_seat_handle_modifiers(void *data, struct wl_keyboard *keyboard,
+ss_seat_handle_modifiers(void *data, struct wl_keyboard *wl_keyboard,
 			 uint32_t serial_in, uint32_t mods_depressed,
 			 uint32_t mods_latched, uint32_t mods_locked,
 			 uint32_t group)
 {
 	struct ss_seat *seat = data;
-	struct weston_compositor *c = seat->output->output->compositor;
+	struct weston_compositor *c = seat->base.compositor;
+	struct weston_keyboard *keyboard;
 	uint32_t serial_out;
 
 	/* If we get a key event followed by a modifier event with the
@@ -272,7 +282,8 @@ ss_seat_handle_modifiers(void *data, struct wl_keyboard *keyboard,
 	else
 		serial_out = wl_display_next_serial(c->wl_display);
 
-	xkb_state_update_mask(seat->base.keyboard->xkb_state.state,
+	keyboard = weston_seat_get_keyboard(&seat->base);
+	xkb_state_update_mask(keyboard->xkb_state.state,
 			      mods_depressed, mods_latched,
 			      mods_locked, 0, 0, group);
 	notify_modifiers(&seat->base, serial_out);
@@ -405,7 +416,7 @@ shared_output_get_shm_buffer(struct shared_output *so)
 	    so->shm.height != height) {
 
 		/* Destroy free buffers */
-		wl_list_for_each_safe(sb, bnext, &so->shm.free_buffers, link)
+		wl_list_for_each_safe(sb, bnext, &so->shm.free_buffers, free_link)
 			ss_shm_buffer_destroy(sb);
 
 		/* Orphan in-use buffers so they get destroyed */
@@ -966,7 +977,7 @@ shared_output_destroy(struct shared_output *so)
 
 	wl_list_for_each_safe(buffer, bnext, &so->shm.buffers, link)
 		ss_shm_buffer_destroy(buffer);
-	wl_list_for_each_safe(buffer, bnext, &so->shm.free_buffers, link)
+	wl_list_for_each_safe(buffer, bnext, &so->shm.free_buffers, free_link)
 		ss_shm_buffer_destroy(buffer);
 
 	wl_display_disconnect(so->parent.display);
@@ -982,13 +993,18 @@ shared_output_destroy(struct shared_output *so)
 }
 
 static struct shared_output *
-weston_output_share(struct weston_output *output,
-		    const char *path, char *const argv[])
+weston_output_share(struct weston_output *output, const char* command)
 {
 	int sv[2];
 	char str[32];
 	pid_t pid;
 	sigset_t allsigs;
+	char *const argv[] = {
+	  "/bin/sh",
+	  "-c",
+	  (char*)command,
+	  NULL
+	};
 
 	if (socketpair(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0, sv) < 0) {
 		weston_log("weston_output_share: socketpair failed: %m\n");
@@ -1025,7 +1041,7 @@ weston_output_share(struct weston_output *output,
 		snprintf(str, sizeof str, "%d", sv[1]);
 		setenv("WAYLAND_SERVER_SOCKET", str, 1);
 
-		execv(path, argv);
+		execv(argv[0], argv);
 		weston_log("weston_output_share: exec failed: %m\n");
 		abort();
 	} else {
@@ -1052,42 +1068,49 @@ weston_output_find(struct weston_compositor *c, int32_t x, int32_t y)
 }
 
 static void
-share_output_binding(struct weston_seat *seat, uint32_t time, uint32_t key,
+share_output_binding(struct weston_keyboard *keyboard, uint32_t time, uint32_t key,
 		     void *data)
 {
 	struct weston_output *output;
-	const char *path = BINDIR "/weston";
+	struct weston_pointer *pointer;
+	struct screen_share *ss = data;
 
-	if (!seat->pointer) {
+	pointer = weston_seat_get_pointer(keyboard->seat);
+	if (!pointer) {
 		weston_log("Cannot pick output: Seat does not have pointer\n");
 		return;
 	}
 
-	output = weston_output_find(seat->compositor,
-				    wl_fixed_to_int(seat->pointer->x),
-				    wl_fixed_to_int(seat->pointer->y));
+	output = weston_output_find(pointer->seat->compositor,
+				    wl_fixed_to_int(pointer->x),
+				    wl_fixed_to_int(pointer->y));
 	if (!output) {
 		weston_log("Cannot pick output: Pointer not on any output\n");
 		return;
 	}
 
-	char *const argv[] = {
-		"weston",
-		"--backend=rdp-backend.so",
-		"--shell=fullscreen-shell.so",
-		"--no-clients-resize",
-		NULL
-	};
-
-	weston_output_share(output, path, argv);
+	weston_output_share(output, ss->command);
 }
 
 WL_EXPORT int
 module_init(struct weston_compositor *compositor,
 	    int *argc, char *argv[])
 {
+	struct screen_share *ss;
+	struct weston_config_section *section;
+
+	ss = zalloc(sizeof *ss);
+	if (ss == NULL)
+		return -1;
+	ss->compositor = compositor;
+
+	section = weston_config_get_section(compositor->config, "screen-share",
+					    NULL, NULL);
+
+	weston_config_section_get_string(section, "command", &ss->command, "");
+
 	weston_compositor_add_key_binding(compositor, KEY_S,
 				          MODIFIER_CTRL | MODIFIER_ALT,
-					  share_output_binding, compositor);
+					  share_output_binding, ss);
 	return 0;
 }
