@@ -1,23 +1,26 @@
 /*
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -28,6 +31,7 @@
 #include <fcntl.h>
 
 #include "xwayland.h"
+#include "shared/helpers.h"
 
 static int
 writable_callback(int fd, uint32_t mask, void *data)
@@ -107,10 +111,14 @@ weston_wm_get_incr_chunk(struct weston_wm *wm)
 				  0x1fffffff /* length */);
 
 	reply = xcb_get_property_reply(wm->conn, cookie, NULL);
+	if (reply == NULL)
+		return;
 
 	dump_property(wm, wm->atom.wl_selection, reply);
 
 	if (xcb_get_property_value_length(reply) > 0) {
+		/* reply's ownership is transfered to wm, which is responsible
+		 * for freeing it */
 		weston_wm_write_property(wm, reply);
 	} else {
 		weston_log("transfer complete\n");
@@ -179,6 +187,8 @@ weston_wm_get_selection_targets(struct weston_wm *wm)
 				  4096 /* length */);
 
 	reply = xcb_get_property_reply(wm->conn, cookie, NULL);
+	if (reply == NULL)
+		return;
 
 	dump_property(wm, wm->atom.wl_selection, reply);
 
@@ -188,8 +198,10 @@ weston_wm_get_selection_targets(struct weston_wm *wm)
 	}
 
 	source = malloc(sizeof *source);
-	if (source == NULL)
+	if (source == NULL) {
+		free(reply);
 		return;
+	}
 
 	wl_signal_init(&source->base.destroy_signal);
 	source->base.accept = data_source_accept;
@@ -230,13 +242,17 @@ weston_wm_get_selection_data(struct weston_wm *wm)
 
 	reply = xcb_get_property_reply(wm->conn, cookie, NULL);
 
-	if (reply->type == wm->atom.incr) {
-		dump_property(wm, wm->atom.wl_selection, reply);
+	dump_property(wm, wm->atom.wl_selection, reply);
+
+	if (reply == NULL) {
+		return;
+	} else if (reply->type == wm->atom.incr) {
 		wm->incr = 1;
 		free(reply);
 	} else {
-		dump_property(wm, wm->atom.wl_selection, reply);
 		wm->incr = 0;
+		/* reply's ownership is transfered to wm, which is responsible
+		 * for freeing it */
 		weston_wm_write_property(wm, reply);
 	}
 }
